@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2001 Michele Comitini <mcm@initd.net>
+ * Copyright (C) 2001 Federico Di Gregorio <fog@debian.org>
  *
  * This file is part of the psycopg module.
  *
@@ -20,21 +21,23 @@
 
 /*
  * cursor.c -- defines the cursor object DBAPI 2.0
- * $Id: cursor.c,v 1.95.2.6 2001/09/03 11:09:57 fog Exp $
+ * $Id: cursor.c,v 1.102.2.2 2002/01/17 18:47:55 fog Exp $
  */
 
-#include <stdio.h>
 #include <assert.h>
-#include <Python.h>
-#include <structmember.h>
-#include <libpq-fe.h>
-#include <pthread.h>
-#include "module.h"
+
 #include "config.h"
+#include "module.h"
 #include "typemod.h"
 
+/* with some versions of postgres we need to include postgres.h to
+   get the correct definition for InvalidOid */
+#ifndef InvalidOid
+#include <postgres.h>
+#endif
 
-/**** UTILITY FUNCTIONS (callable from C) ****/
+
+/**** UTILITY FUNCTIONS (callable from C only) ****/
 
 /* pgconn_notice_*() - callbacks to process NOTICEs  */
 
@@ -77,8 +80,8 @@ _psyco_curs_getout(cursobject *self)
             if (self == (cursobject *)tmpobj) {
                 Dprintf("_psyco_curs_getout(): found myself in cursor list\n");
                 PySequence_DelItem(self->conn->cursors, i);
-                Dprintf("_psyco_curs_getout(): cursor at %p removed from list, "
-                        "refcnt = %d\n", tmpobj, tmpobj->ob_refcnt);
+                Dprintf("_psyco_curs_getout(): cursor at %p removed from "
+                        "list, refcnt = %d\n", tmpobj, tmpobj->ob_refcnt);
                 break;
             }
         }
@@ -145,7 +148,8 @@ dispose_pgconn(cursobject *self)
         cpgconn = PyCObject_FromVoidPtr((void *)self->keeper, NULL);
         Dprintf("dispose_pgconn(): appending keeper PyCObject to the list\n");
         PyList_Append(self->conn->avail_conn, cpgconn);
-        Dprintf("dispose_pgconn(): cpgconn->refcnt = %d\n", cpgconn->ob_refcnt);
+        Dprintf("dispose_pgconn(): cpgconn->refcnt = %d\n",
+                cpgconn->ob_refcnt);
         Py_DECREF(cpgconn);
         pthread_mutex_unlock(&(self->conn->lock));
     }
@@ -174,7 +178,7 @@ _psyco_curs_destroy(cursobject *self)
     /* remove the cursor from the connection list */
     if (self->conn) {
         pthread_mutex_lock(&(self->conn->lock));
-        //Py_INCREF(self);
+        /* Py_INCREF(self); */
         _psyco_curs_getout(self);
         pthread_mutex_unlock(&(self->conn->lock));
     }
@@ -233,7 +237,7 @@ pgconn_resolve_critical(cursobject *self)
         /* we don't want to destroy this cursor but just to close it, so we
            incref it to prevent the garbage collector to invoke the destroy
            function */
-        //Py_INCREF(self);
+        /* Py_INCREF(self); */
         psyco_curs_close(self, NULL);
         Dprintf("pgconn_resolve_critical(): resolved\n");
     }
@@ -262,12 +266,6 @@ commit_pgconn(cursobject *self)
         return 0;
     
     assert(self->pgconn);
-    /* if (PQstatus(self->pgconn) != CONNECTION_OK){
-        Dprintf("commit_pgconn(): connection NOT OK\n");
-        PyErr_SetString(OperationalError, PQerrorMessage(self->pgconn));
-        goto cleanup;
-    }
-    Dprintf("commit_pgconn(): connection at %p is OK\n", self->pgconn); */
     
     pgres = PQexec(self->pgconn, query);
     if (pgres == NULL) {
@@ -315,13 +313,7 @@ begin_pgconn(cursobject *self)
         return 0;
     
     assert(self->pgconn);
-    /* if (PQstatus(self->pgconn) != CONNECTION_OK){
-        Dprintf("begin_pgconn(): connection NOT OK\n");
-        PyErr_SetString(OperationalError, PQerrorMessage(self->pgconn));
-        goto cleanup;
-    }
-    Dprintf("begin_pgconn(): connection at %p is OK\n", self->pgconn); */
-    
+
     pgres = PQexec(self->pgconn, query);
     if (pgres == NULL) {
         pgconn_set_critical(self);
@@ -418,10 +410,9 @@ alloc_keeper(connobject *conn)
     const char *datestyle = "SET DATESTYLE TO 'ISO'";
     
     Dprintf("alloc_keeper(): opening new postgresql connection\n");
-    Dprintf("alloc_keeper(): dsn = %s\n", conn->dsn);
-    Py_BEGIN_ALLOW_THREADS;
+    Dprintf("alloc_keeper(): dsn = %s (%p)\n", conn->dsn, conn->dsn);
+    
     pgconn = PQconnectdb(conn->dsn);
-    Py_END_ALLOW_THREADS;
     Dprintf("alloc_keeper(): new posgresql connection at %p\n", pgconn);
         
     if (pgconn == NULL)
@@ -507,7 +498,6 @@ request_pgconn(cursobject *self)
     connobject *owner_conn;
     connkeeper *keeper;
     int ncursor_conn, navail_conn, nopen_conn;
-    int result;
 
     /* if we return an error, we want the cursor's postgres connection to
        be set to NULL */
@@ -576,22 +566,6 @@ request_pgconn(cursobject *self)
     self->keeper = keeper;
     self->pgconn = keeper->pgconn;
     
-    /* start the transaction (FIXME: are the locks really necessary here?
-       we are creating the cursor, so it is *not yet* in any list and nobody
-       can reach this new keeper... mmm...) */
-    //pthread_mutex_lock(&(self->keeper->lock));
-    Py_BEGIN_ALLOW_THREADS;
-    result = begin_pgconn(self);
-    Py_END_ALLOW_THREADS;
-    if (result < 0) {
-        //pthread_mutex_unlock(&(self->keeper->lock));
-        free(keeper);
-        self->pgconn = NULL;
-        self->keeper = NULL;
-        return -1;
-    }
-    //pthread_mutex_unlock(&(self->keeper->lock));
-    
     Dprintf("request_pgconn(): cursor at %p using keeper at %p, "
             "connection at %p\n", self, keeper, keeper->pgconn);
     return 0;
@@ -612,19 +586,20 @@ curs_switch_autocommit(cursobject *self, long int ac)
     
     /* if the cursor was not in autocommit and we set autocommit we need to
        ABORT the transaction */
-    if (((cursobject*)self)->autocommit == 0 && ac != 0) {
+    if (self->autocommit == 0 && ac != 0) {
         if (abort_pgconn(self) < 0) goto error;
-        ((cursobject*)self)->autocommit = 1;
+        self->autocommit = 1;
     }
 
-    /* if the cursor was in autocommit and we switch it off, we need to BEGIN
-       a new transaction */
-    if (((cursobject*)self)->autocommit != 0 && ac == 0) {
-        ((cursobject*)self)->autocommit = 0;
-        if (begin_pgconn(self) < 0) goto error;
+    /* else we simply set autocommit to 0 */
+    if (self->autocommit != 0 && ac == 0) {
+        self->autocommit = 0;
     }
 
  error:
+    Dprintf("curs_switch_autocommit(): pgconn = %p, autocommit = %d, "
+            "status = %d\n", self->pgconn, self->autocommit,
+            self->keeper->status);
     pthread_mutex_unlock(&(self->keeper->lock));
     Py_END_ALLOW_THREADS;
 }
@@ -656,7 +631,6 @@ psyco_curs_reset(cursobject *self, int resetconn)
         pthread_mutex_lock(&(self->keeper->lock));
         Py_BEGIN_ALLOW_THREADS;
         abort_pgconn(self);
-        begin_pgconn(self);
         pthread_mutex_unlock(&(self->keeper->lock));
         Py_END_ALLOW_THREADS;
     }
@@ -685,7 +659,7 @@ psyco_curs_abort(cursobject *self, PyObject *args) {
     Py_BEGIN_ALLOW_THREADS;
     
     if (abort_pgconn(self) < 0) goto cleanup;
-    if (begin_pgconn(self) < 0) goto cleanup;
+
     Py_INCREF(Py_None);
     result = Py_None;
 
@@ -718,7 +692,6 @@ psyco_curs_commit(cursobject *self, PyObject *args) {
     Py_BEGIN_ALLOW_THREADS;
     
     if (commit_pgconn(self) < 0) goto cleanup;
-    if (begin_pgconn(self) < 0) goto cleanup;
     Py_INCREF(Py_None);
     result = Py_None;
 
@@ -737,7 +710,7 @@ psyco_curs_commit(cursobject *self, PyObject *args) {
 static PyObject *
 _psyco_curs_execute(cursobject *self, char *query)
 {
-    int pgstatus;
+    int pgstatus, old_keeper_status;
     
     /* even if we fail, we remove any information about the previous query */
     psyco_curs_reset(self, 0);
@@ -757,6 +730,7 @@ _psyco_curs_execute(cursobject *self, char *query)
     pthread_mutex_lock(&(self->keeper->lock));
     Py_BEGIN_ALLOW_THREADS;
     Dprintf("_psyco_curs_execute(): query = >%s<\n", query);
+    begin_pgconn(self);
     self->pgres = PQexec(self->pgconn, query);
     Dprintf("_psyco_curs_execute(): query executed\n");
 
@@ -766,7 +740,8 @@ _psyco_curs_execute(cursobject *self, char *query)
        connection, it better has to sync with us. because us and him are
        about to work on the same keeper, setting status to LOCKED is like
        shifting *our* execution time a little bit in the future... */
-    self->keeper->status = KEEPER_LOCKED;   
+    old_keeper_status = self->keeper->status;
+    self->keeper->status = KEEPER_LOCKED;
     pthread_mutex_unlock(&(self->keeper->lock));
     Py_END_ALLOW_THREADS;
 
@@ -798,14 +773,15 @@ _psyco_curs_execute(cursobject *self, char *query)
     else if (pgstatus == PGRES_TUPLES_OK)
     {
         int i, pgnfields = PQnfields(self->pgres);
-
+        int pgbintuples = PQbinaryTuples(self->pgres);
+        
         self->notuples = 0;
         self->rowcount = PQntuples(self->pgres);
         
         Dprintf("_psyco_curs_execute(): got %ld tuples\n", self->rowcount);
 
         /* create the tuple for description and typecasting */
-        Py_XDECREF(self->description);
+        Py_XDECREF(self->description); Py_XDECREF(self->casts);
         self->description = PyTuple_New(pgnfields);
         self->casts = PyTuple_New(pgnfields);
         self->columns = pgnfields;
@@ -819,21 +795,30 @@ _psyco_curs_execute(cursobject *self, char *query)
             PyTuple_SET_ITEM(self->description, i, dtitem);
 
             /* fill the right cast function by accessing the global dictionary
-               of casting objects.  If we got binary tuples or no defined cast
-               use the default one.
+               of casting objects.  If we got no defined cast use the default
+               one.
             */
-            if (PQbinaryTuples(self->pgres) ||
-                !(cast = PyDict_GetItem(psyco_types, type))) {
-                Dprintf("_psyco_curs_execute: cast not found, using default\n");
+            if (!(cast = PyDict_GetItem(psyco_types, type))) {
+                Dprintf("_psyco_curs_execute: cast %d not found, using "
+                        "default\n", PQftype(self->pgres,i));
                 cast = psyco_default_cast;
             }
-            Dprintf("_psyco_curs_execute: using cast at %p\n", cast);
+            /* else if we got binary tuples and if we got a field that
+               is binary use the default cast.
+            */
+            else if (pgbintuples && cast == psyco_binary_cast) {
+                Dprintf("_psyco_curs_execute: Binary cursor and binary field: "
+                        "%i using default cast\n", PQftype(self->pgres,i));
+                cast = psyco_default_cast;
+            }
+            Dprintf("_psyco_curs_execute: using cast at %p for type %d\n",
+                    cast, PQftype(self->pgres,i));
             Py_INCREF(cast);
             PyTuple_SET_ITEM(self->casts, i, cast);
             
             /* fill the other fields */
             PyTuple_SET_ITEM(dtitem, 0,
-                            PyString_FromString(PQfname(self->pgres,i)));
+                             PyString_FromString(PQfname(self->pgres,i)));
             PyTuple_SET_ITEM(dtitem, 1, type);
             
             /* FIXME: display size is the maximum size of this field in result
@@ -847,7 +832,8 @@ _psyco_curs_execute(cursobject *self, char *query)
             PyTuple_SET_ITEM(dtitem, 2, Py_None);
             
             /* size on the backend */
-            PyTuple_SET_ITEM(dtitem, 3, PyInt_FromLong(PQfsize(self->pgres, i)));
+            PyTuple_SET_ITEM(dtitem, 3,
+                             PyInt_FromLong(PQfsize(self->pgres, i)));
 
             /* FIXME: scale??? */
             Py_INCREF(Py_None);
@@ -865,7 +851,10 @@ _psyco_curs_execute(cursobject *self, char *query)
     
     else {  /* PGRES_COMMAND_OK */
         Dprintf("_psyco_curs_execute(): command returned OK (no tuples)\n");
+        /* FIXME: why we read a value that we _know_ being 0? */
         self->rowcount = atoi(PQcmdTuples(self->pgres));
+        /* try to obtain the oid of an insert */
+        self->last_oid = PQoidValue(self->pgres);
         Py_XDECREF(self->description);
         Py_INCREF(Py_None);
         self->description = Py_None;
@@ -876,7 +865,7 @@ _psyco_curs_execute(cursobject *self, char *query)
 
     /* here we replace status with its default value */
     pthread_mutex_lock(&(self->keeper->lock));
-    self->keeper->status = KEEPER_BEGIN;
+    self->keeper->status = old_keeper_status;
     pthread_mutex_unlock(&(self->keeper->lock));
     Py_INCREF(Py_None);
     return Py_None;
@@ -885,8 +874,8 @@ _psyco_curs_execute(cursobject *self, char *query)
     Dprintf("_psyco_curs_execute(): error, resetting connection\n");
     pthread_mutex_lock(&(self->keeper->lock));
     Py_BEGIN_ALLOW_THREADS;
-    self->keeper->status = KEEPER_BEGIN;
-    if (!abort_pgconn(self)) begin_pgconn(self);
+    self->keeper->status = old_keeper_status;
+    abort_pgconn(self);
     pthread_mutex_unlock(&(self->keeper->lock));
     Py_END_ALLOW_THREADS;
     EXC_IFCRITICAL(self);
@@ -904,13 +893,17 @@ _mogrify_dict(PyObject *dict)
     new_dict = PyDict_New();
 
     while (PyDict_Next(dict, &pos, &key, &value)) {
-        //Py_INCREF(key);
+        /* Py_INCREF(key); */
         if (PyString_Check(value)) {
             PyDict_SetItem(new_dict, key, new_psyco_quotedstringobject(value));
         }
+        else if (value == Py_None) {
+            PyDict_SetItem(new_dict, key, PyString_FromString("NULL"));
+            Py_DECREF(Py_None);
+        }
         else {
             PyDict_SetItem(new_dict, key, value);
-            //Py_INCREF(value);
+            /* Py_INCREF(value); */
         }
     }
 
@@ -924,7 +917,6 @@ _mogrify_seq(PyObject *seq)
     PyObject *new_seq, *value;
 
     len = PyObject_Length(seq);
-
     new_seq = PyTuple_New(len);
 
     for (i=0; i<len; i++) {
@@ -932,6 +924,10 @@ _mogrify_seq(PyObject *seq)
         if (PyString_Check(value)) {
             PyTuple_SET_ITEM(new_seq, i, new_psyco_quotedstringobject(value));
         }
+        else if (value == Py_None) {
+            PyTuple_SET_ITEM(new_seq, i, PyString_FromString("NULL"));
+            Py_DECREF(Py_None);
+        }
         else {
             PyTuple_SET_ITEM(new_seq, i, value);
             Py_INCREF(value);
@@ -1124,9 +1120,9 @@ psyco_curs_executemany(cursobject *self, PyObject *args)
 
     for (i = 0; i < PyTuple_Size(parm_seq); i++) {
         seq_item = PySequence_GetItem(parm_seq, i); 
-        if (!PyDict_Check(seq_item)) {
+        if (!PyDict_Check(seq_item) && !PyTuple_Check(seq_item)) {
             PyErr_SetString(PyExc_TypeError,
-                            "arg 2 must be a dictionary sequence");
+                            "arg 2 must be a dictionary or tuple sequence");
             return NULL;
         }
         Py_INCREF(seq_item);
@@ -1154,11 +1150,7 @@ psyco_curs_fetchone(cursobject *self, PyObject *args)
 {
     int coln, i;
     PyObject *res;
-    register PGresult *r;
-    
-    /* speed up argument passing by preallocating tuple */
-    static PyObject *arg = NULL;
-    if (!arg) arg = PyTuple_New(1);
+    PGresult *r;
     
     PARSEARGS(args);
     EXC_IFCLOSED(self);
@@ -1185,10 +1177,14 @@ psyco_curs_fetchone(cursobject *self, PyObject *args)
             char *s = PQgetvalue(r, self->row, i);
             int l = PQgetlength(r, self->row, i);
             PyObject *str = PyString_FromStringAndSize(s, l);
-            Dprintf("psyco_curs_fetchone: row %ld, element %d, len %i", self->row, i, l);
+            PyObject *arg = PyTuple_New(1);
+
+            Dprintf("psyco_curs_fetchone: row %ld, element %d, len %i\n",
+                    self->row, i, l);
             PyTuple_SET_ITEM(arg, 0, str);
             val = PyObject_CallObject(PyTuple_GET_ITEM(self->casts, i), arg);
-            Py_DECREF(str);
+            
+            Py_DECREF(arg);
         }
         if (val) {
             PyTuple_SET_ITEM(res, i, val);
@@ -1304,7 +1300,7 @@ psyco_curs_fetchall(cursobject *self, PyObject *args)
 }
 
 
-/* psyco_curs_dictfetchone() - fetch onw row of data ina  dictionary */
+/* psyco_curs_dictfetchone() - fetch onw row of data in a dictionary */
 
 static char psyco_curs_dictfetchone__doc__[] = 
 "Fetch the next row of a query result set, returning a "
@@ -1335,10 +1331,12 @@ psyco_curs_dictfetchone(cursobject *self, PyObject *args)
                 "value.ob_refcnt = %d\n", name->ob_refcnt, value->ob_refcnt);
     }
     
+    Dprintf("psyco_curs_dictfetchone(): res.ob_refcnt = %d\n", res->ob_refcnt);
     Py_DECREF(res);
     return dict;
 }
 
+/* cosimo 888268 */
 
 /* psyco_curs_dictfetchmany() - fetch many rows into dictionaries */
    
@@ -1519,6 +1517,30 @@ psyco_curs_autocommit(cursobject *self, PyObject *args)
 }
 
 
+/* psyco_curs_lastoid() - return the last oid from an insert or None */
+
+static char psyco_curs_lastoid__doc__[] =
+"Return the last oid from an insert or None.""";
+
+static PyObject *
+psyco_curs_lastoid(cursobject *self, PyObject *args)
+{    
+    PARSEARGS(args);
+    EXC_IFCRITICAL(self);
+    
+    Dprintf("psyco_curs_lastoid(): self->last_oid = %d on %p\n",
+            self->last_oid, self);
+
+    if (self->last_oid != InvalidOid) {
+        return PyInt_FromLong((long)self->last_oid);
+    }
+    else {
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+}
+
+
 /**** CURSOR OBJECT DEFINITION ****/
 
 /* object methods list */
@@ -1556,6 +1578,8 @@ static struct PyMethodDef psyco_curs_methods[] = {
      METH_VARARGS, psyco_curs_nextset__doc__},
     {"autocommit", (PyCFunction)psyco_curs_autocommit,
      METH_VARARGS, psyco_curs_autocommit__doc__},
+    {"lastoid", (PyCFunction)psyco_curs_lastoid,
+     METH_VARARGS, psyco_curs_lastoid__doc__},
     {NULL, NULL}
 };
 
@@ -1608,7 +1632,7 @@ psyco_curs_destroy(cursobject *self)
 
     Dprintf("psyco_curs_destroy(): cursor at %p destroyed, refcnt = %d\n",
             self, self->ob_refcnt);
-    PyMem_DEL(self);
+    PyObject_Del(self);
 }
 
 
@@ -1666,6 +1690,7 @@ new_psyco_cursobject(connobject *conn, connkeeper *keeper)
     self->arraysize = 1;
     self->rowcount = -1;
     self->closed = 0;
+    self->last_oid = InvalidOid;
     self->autocommit = conn->autocommit ? 1 : 0;
     self->casts = NULL;
     self->notice = NULL;
@@ -1703,4 +1728,3 @@ new_psyco_cursobject(connobject *conn, connkeeper *keeper)
             self, self->ob_refcnt);
     return self;
 }
-
