@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2001 Michele Comitini <mcm@initd.net>
+ * Copyright (C) 2001 Federico Di Gregorio <fog@debian.org>
  *
  * This file is part of the psycopg module.
  *
@@ -20,20 +21,20 @@
 
 /*
  * module.c -- defines the module interface to DBAPI 2.0
- * $Id: module.c,v 1.36.2.3 2001/09/03 11:09:57 fog Exp $
+ * $Id: module.c,v 1.38 2001/10/29 17:57:58 fog Exp $
  */
 
 #include <time.h>
-#include <Python.h>
-#include <structmember.h>
-#include <mxDateTime.h>
-#include <libpq-fe.h>
 
 #include "config.h"
 #include "module.h"
 #include "typeobj.h"
 #include "typemod.h"
 
+/* included after pthread.h to work on MacOS X */
+#include <Python.h>
+#include <structmember.h>
+
 
 /**** MODULE FUNCTIONS ****/
 
@@ -121,6 +122,9 @@ static PyMethodDef psycopgMethods[] = {
     {"DateFromTicks", (PyCFunction)psyco_DateFromTicks, METH_VARARGS},
     {"TimeFromTicks", (PyCFunction)psyco_TimeFromTicks, METH_VARARGS},
     {"TimestampFromTicks", (PyCFunction)psyco_TimestampFromTicks, METH_VARARGS},
+    {"DateFromMx", (PyCFunction)psyco_DateFromMx, METH_VARARGS},
+    {"TimeFromMx", (PyCFunction)psyco_TimeFromMx, METH_VARARGS},
+    {"TimestampFromMx", (PyCFunction)psyco_TimestampFromMx, METH_VARARGS},
     {"Binary", (PyCFunction)psyco_Binary, METH_VARARGS},
     {"QuotedString", (PyCFunction)psyco_QuotedString, METH_VARARGS},
     {NULL,  NULL}
@@ -136,59 +140,27 @@ mxDateTimeModule_APIObject *mxDateTimeP;
 void
 initpsycopg(void)
 {
-	PyObject *m, *d, *msys, *v;
-    PyObject *datetime_path, *mod, *api, *api_obj;
+	PyObject *m, *d;
 
-    /* import the sys module */
-    msys = PyImport_ImportModule("sys");
-    if (!msys) {
-        return;
-    }
-    v = PyObject_GetAttrString(msys, "path");
-    datetime_path = PyString_FromString(MXDATETIME_LOCATION);
-    if (!datetime_path) {
-        return;
-    }
-    if (PyList_Insert(v, 0, datetime_path) == -1) {
-        Py_DECREF(datetime_path);
-        return;
-    }
-    mod = PyImport_ImportModule(MXDATETIME_API_MODULE);
-    if (mod == NULL) {
-        return ;
-    }
-    api_obj = PyObject_GetAttrString(mod, MXDATETIME_MODULE"API");
-    if (api_obj == NULL) {
-        Py_DECREF(mod);
-        return;
-    }
-    Py_DECREF(mod);
-    Dprintf("initpsycopg(): API object found\n");
-    api = PyCObject_AsVoidPtr(api_obj);
-    if (api == NULL) {
-        Py_DECREF(mod);
-        Py_DECREF(v);
-        return;
-    }
-    memcpy(&mxDateTime, api, sizeof(mxDateTime));
+    mxDateTime_ImportModuleAndAPI();
     mxDateTimeP = &mxDateTime;
+    Dprintf("initpsycopg: mxDateTime module imported at %p\n", mxDateTimeP); 
     
-    PySequence_DelItem(v, 0);
-    Py_DECREF(datetime_path);
-    Py_DECREF(v);
-
     /* init module and grab module namespace (dictionary) */
 	m = Py_InitModule("psycopg", psycopgMethods);
     d = PyModule_GetDict(m);
-
+    Dprintf("initpsycopg: module initialized\n");
+    
     /* DBAPI compliance module parameters */
     PyDict_SetItemString(d, "__version__", PyString_FromString(VERSION));
     PyDict_SetItemString(d, "apilevel", PyString_FromString(APILEVEL));
     PyDict_SetItemString(d, "threadsafety", PyInt_FromLong(THREADSAFETY));
     PyDict_SetItemString(d, "paramstyle", PyString_FromString(PARAMSTYLE));
-
+    Dprintf("initpsycopg: parameters initialized\n");
+ 
     /* create the default DBAPITypeObject dictionary */
     psyco_init_types(d);
+    Dprintf("initpsycopg: types initialized\n");
     
 	/* exceptions of this module */
 	Error = PyErr_NewException("psycopg.Error", PyExc_StandardError,NULL);
@@ -227,4 +199,6 @@ initpsycopg(void)
 	NotSupportedError =
         PyErr_NewException("psycopg.NotSupportedError", DatabaseError, NULL);
 	PyDict_SetItemString(d, "NotSupportedError", NotSupportedError);
+
+    Dprintf("initpsycopg: exceptions initialized\n");
 }
