@@ -20,15 +20,11 @@
 
 /*
  * typemod.c -- types available to the psycopg module
- * $Id: typemod.c,v 1.2.2.5 2001/09/03 11:09:57 fog Exp $
+ * $Id: typemod.c,v 1.9 2001/10/31 16:33:33 fog Exp $
  */
 
-#include <stdio.h>
-#include <time.h>
-#include <Python.h>
-#include <structmember.h>
-#include <libpq-fe.h>
-#include <mxDateTime.h>
+#include <unistd.h>
+#include <assert.h>
 
 #include "module.h"
 #include "typemod.h"
@@ -85,7 +81,7 @@ static void
 psyco_DateTimeObject_destroy(psyco_DateTimeObject *self)
 {
     Py_XDECREF(self->datetime);
-    PyMem_DEL(self);
+    PyObject_Del(self);
     Dprintf("DateTimeObject at %p destroyed\n", self);
 }
 
@@ -182,7 +178,7 @@ static void
 psyco_BufferObject_destroy(psyco_BufferObject *self)
 {
     Py_DECREF(self->buffer);
-    PyMem_DEL(self);
+    PyObject_Del(self);
     Dprintf("psyco_BufferObject_destroy(): bufferobject at %p destroyed\n", self);
 }
 
@@ -220,35 +216,44 @@ static PyObject *
 new_psyco_bufferobject(PyObject *buffer)
 {
     psyco_BufferObject *obj;
-    char *original, *quoted, *chptr;
-    int i, len;
+    unsigned char *original, *quoted, *chptr, *newptr;
+    int i, len, space, new_space;
 
-    /* FIXME: do we really need to check? psyco_Binary() already did
-       typechecking and this function can't be called without passing by
-       psyco_Binary(), right?
-
-    if (!PyString_Check(buffer)) {
-        PyErr_SetString(PyExc_TypeError, "argument should be string");
-        return NULL;
-    }
-    */
     
     obj = PyObject_NEW(psyco_BufferObject, &psyco_BufferObject_Type);
     if (obj == NULL) return NULL;
 
     original = PyString_AS_STRING(buffer);
     len = PyString_GET_SIZE(buffer);
-    
-    quoted = (char*)calloc(len*5+2, 1);
+    space = len + 2;
+
+    Py_BEGIN_ALLOW_THREADS;
+
+    quoted = (char*)calloc(space, sizeof(char));
     if (quoted == NULL) return NULL;
 
     chptr = quoted;
     *chptr = '\'';
     chptr++;
 
-    for (i=0; i<len; i++) {
+    for (i=0; i < len; i++) {
+        if (chptr - quoted > space - 6) {
+            new_space  =  space * ((space) / (i + 1)) + 2 + 6;
+            if (new_space - space < 1024) space += 1024;
+            else space = new_space;
+            newptr = (char *)realloc(quoted, space);
+            if (newptr == NULL) {
+                free(quoted);
+                return NULL;
+            }
+            /*chptr has to be moved to the new location*/
+            chptr = newptr + (chptr - quoted);
+            quoted = newptr;
+            Dprintf("new_psyco_bufferobject(): reallocated %i bytes at %p \n",
+                    space, quoted);
+        }
         if (original[i]) {
-            if (original[i] > ' ') {
+            if (original[i] >= ' ') {
                 if (original[i] == '\'') {
                     *chptr = '\\';
                     chptr++;
@@ -260,7 +265,7 @@ new_psyco_bufferobject(PyObject *buffer)
                     chptr += 4;
                 }
                 else {
-                    /* leave it as it is if > SPACE */
+                    /* leave it as it is if >= ' ' */
                     *chptr = original[i];
                     chptr++;
                 }
@@ -287,11 +292,11 @@ new_psyco_bufferobject(PyObject *buffer)
         }
     }
     *chptr = '\'';
-    chptr++;
-    *chptr = '\0';
-    
+
+    Py_END_ALLOW_THREADS;
+
     Dprintf("new_psyco_bufferobject(): quoted = %s\n", quoted);
-    obj->buffer = PyString_FromString(quoted);
+    obj->buffer = PyString_FromStringAndSize(quoted, chptr - quoted + 1 );
     free(quoted);
     
     Dprintf("new_psyco_bufferobject: object created at %p\n", obj);
@@ -334,7 +339,8 @@ psyco_QuotedStringObject_getattr(psyco_BufferObject *self, char *name)
     rv = PyMember_Get((char *)self, psyco_QuotedStringObject_memberlist, name);
     if (rv) return rv;
     PyErr_Clear();
-    return Py_FindMethod(psyco_QuotedStringObject_methods, (PyObject *)self, name);
+    return Py_FindMethod(psyco_QuotedStringObject_methods,
+                         (PyObject *)self, name);
 }
 
 
@@ -342,7 +348,7 @@ static void
 psyco_QuotedStringObject_destroy(psyco_QuotedStringObject *self)
 {
     Py_DECREF(self->buffer);
-    PyMem_DEL(self);
+    PyObject_Del(self);
     Dprintf("psyco_QuotedStringObject_destroy(): "
             "quotedstringobject at %p destroyed\n", self);
 }
@@ -384,8 +390,7 @@ new_psyco_quotedstringobject(PyObject *buffer)
     char *original, *quoted;
     int i, j, len;
     
-    obj = PyObject_NEW(psyco_QuotedStringObject,
-                       &psyco_QuotedStringObject_Type);
+    obj = PyObject_NEW(psyco_QuotedStringObject, &psyco_QuotedStringObject_Type);
     if (obj == NULL) return NULL;
 
     original = PyString_AS_STRING(buffer);
@@ -413,7 +418,11 @@ new_psyco_quotedstringobject(PyObject *buffer)
     }
     quoted[0] = '\'';
     quoted[j++] = '\'';
+    quoted[j] = '\0';
 
+    /* are we overwriting someone else memory? */
+    assert(j < len*2+2);
+        
     Dprintf("new_psyco_quotedstringobject(): quoted = %s\n", quoted);
 
     obj->buffer = PyString_FromStringAndSize(quoted, j);
@@ -552,6 +561,37 @@ psyco_TimestampFromTicks(PyObject *self, PyObject *args)
     return new_psyco_datetimeobject(datetime, PSYCO_DATETIME_TIMESTAMP);
 }
 
+PyObject *
+psyco_DateFromMx(PyObject *self, PyObject *args)
+{
+    PyObject *o;
+
+    if (!PyArg_ParseTuple(args, "O!", mxDateTimeP->DateTime_Type, &o))
+        return NULL;
+    return new_psyco_datetimeobject(o, PSYCO_DATETIME_DATE);
+}
+
+PyObject *
+psyco_TimeFromMx(PyObject *self, PyObject *args)
+{
+    PyObject *o;
+
+    if (!PyArg_ParseTuple(args, "O!", mxDateTimeP->DateTime_Type, &o))
+        return NULL;
+    return new_psyco_datetimeobject(o, PSYCO_DATETIME_TIME);
+}
+
+PyObject *
+psyco_TimestampFromMx(PyObject *self, PyObject *args)
+{
+    PyObject *o;
+
+    if (!PyArg_ParseTuple(args, "O!", mxDateTimeP->DateTime_Type, &o))
+        return NULL;
+    return new_psyco_datetimeobject(o, PSYCO_DATETIME_TIMESTAMP);
+}
+
+
 PyObject *
 psyco_Binary(PyObject *self, PyObject *args)
 {
