@@ -20,16 +20,10 @@
 
 /*
  * typeobj.c -- define the type objects
- * $Id: typeobj.c,v 1.15.2.2 2001/09/03 11:09:57 fog Exp $
+ * $Id: typeobj.c,v 1.20.2.3 2001/12/29 00:42:32 fog Exp $
  */
 
-#include <stdio.h>
-#include <time.h>
-#include <Python.h>
-#include <structmember.h>
-#include <libpq-fe.h>
-#include <mxDateTime.h>
-
+#include "assert.h"
 #include "module.h"
 #include "typeobj.h"
 
@@ -67,47 +61,193 @@ psyco_STRING_cast(PyObject *s)
 static PyObject *
 psyco_DATE_cast(PyObject *s)
 {
-    struct tm tm;
-    char *tz;
+    int n, y=0, m=0, d=0;
+    int hh=0, mm=0;
+    double ss=0.0;
+
+    char *str = PyString_AsString(s);
     
-    Dprintf("psyco_DATE_cast(): s = %s\n", PyString_AsString(s));
-    memset(&tm, 0, sizeof(struct tm));
-    tz = strptime(PyString_AsString(s), "%Y-%m-%d %H:%M:%S", &tm);
-    Dprintf("psyco_DATE_cast(): date parsed, remains %s\n", tz);
-    /* if tz is null, we try to parse only the date component, else
-       we simply raise an exception */
-    if (tz == NULL) tz = strptime(PyString_AsString(s), "%Y-%m-%d", &tm);
-    if (tz == NULL) {
+    Dprintf("psyco_DATE_cast(): s = %s\n", str);
+    
+    n = sscanf(str, "%d-%d-%d %d:%d:%lf", &y, &m, &d, &hh, &mm, &ss);
+    Dprintf("psyco_DATE_cast(): date parsed, %d components\n", n);
+    
+    if (n != 3 && n != 6) {
         PyErr_SetString(DataError, "unable to parse date");
         return NULL;
     }
-    return mxDateTimeP->DateTime_FromTmStruct(&tm);
+    return mxDateTimeP->DateTime_FromDateAndTime(y, m, d, hh, mm, ss);
 }
 
+#ifdef UNDEF
 static PyObject *
 psyco_TIME_cast(PyObject *s)
 {
     time_t now;
     struct tm tm, *today;
-    char *tz;
+    int n, hh=0, mm=0;
+    double ss=0.0;
+    
+    char *str = PyString_AsString(s);
     
     /* postgres gives us only the time but a DateTime object wants a date too,
        so we use today as the date. on python -> postgres there aren't any
        problems, because postgres simply ignores the extra fields in a date
        and time string. */
-    
-    Dprintf("psyco_TIME_cast(): s = %s\n", PyString_AsString(s));
     time(&now);
     today = gmtime(&now);
     memcpy(&tm, today, sizeof(struct tm));
-    tz = strptime(PyString_AsString(s), "%H:%M:%S", &tm);
-    Dprintf("psyco_TIME_cast(): time parsed, remains %s\n", tz);
-    if (tz == NULL) {
+    
+    Dprintf("psyco_TIME_cast(): s = %s\n", str);
+    
+    n = sscanf(str, "%d:%d:%lf", &hh, &mm, &ss);
+    Dprintf("psyco_TIME_cast(): time parsed, %d components\n", n);
+    Dprintf("psyco_TIME_cast(): hh = %d, mm = %d, ss = %f\n", hh, mm, ss);
+    
+    if (n != 3) {
         PyErr_SetString(DataError, "unable to parse time");
         return NULL;
     }
+    tm.tm_sec = (int)ss;
+    tm.tm_min = mm;
+    tm.tm_hour = hh;
+    
     return mxDateTimeP->DateTime_FromTmStruct(&tm);
 }
+#else
+static PyObject *
+psyco_TIME_cast(PyObject *s)
+{
+    int n, hh=0, mm=0;
+    double ss=0.0;
+    
+    char *str = PyString_AsString(s);
+    
+    /* postgres gives us only the time but a DateTime object wants a date too,
+       so we use the unix epoch as the current date. */
+    
+    Dprintf("psyco_TIME_cast(): s = %s\n", str);
+    
+    n = sscanf(str, "%d:%d:%lf", &hh, &mm, &ss);
+    Dprintf("psyco_TIME_cast(): time parsed, %d components\n", n);
+    Dprintf("psyco_TIME_cast(): hh = %d, mm = %d, ss = %f\n", hh, mm, ss);
+    
+    if (n != 3) {
+        PyErr_SetString(DataError, "unable to parse time");
+        return NULL;
+    }
+
+    return mxDateTimeP->DateTime_FromDateAndTime(1970, 1, 1, hh, mm ,ss);
+}
+#endif
+
+static char *
+skip_until_space(char *s)
+{
+    while (*s && *s != ' ') s++;
+    return s;
+}
+
+static PyObject *
+psyco_INTERVAL_cast(PyObject *s)
+{
+    long years = 0, months = 0, days = 0;
+    double hours = 0.0, minutes = 0.0, seconds = 0.0, v = 0.0, sign = 1.0;
+    int part = 0;
+    char *str;
+
+    str = PyString_AsString(s);
+    Dprintf("psyco_INTERVAL_cast(): s = %s\n", str);
+    Dprintf("hey! postgres guys, do you want to make me crazy?\n");
+
+    while (*str) {
+        switch (*str) {
+
+        case '-':
+            sign = -1.0;
+            break;
+
+        case '0': case '1': case '2': case '3': case '4':
+        case '5': case '6': case '7': case '8': case '9':
+            v = v*10 + (double)*str - (double)'0';
+            Dprintf("psyco_INTERVAL_cast(): v = %f\n", v);
+            break;
+
+        case 'y':
+            if (part == 0) {
+                years = (long)(v*sign);
+                str = skip_until_space(str);
+                Dprintf("psyco_INTERVAL_cast(): years = %ld, rest = %s\n",
+                        years, str);
+                v = 0.0; sign = 1.0; part = 1;
+            }
+            break;
+
+        case 'm':
+            if (part <= 1) {
+                months = (long)(v*sign);
+                str = skip_until_space(str);
+                Dprintf("psyco_INTERVAL_cast(): months = %ld, rest = %s\n",
+                        months, str);
+                v = 0.0; sign = 1.0; part = 2;
+            }
+            break;
+
+        case 'd':
+            if (part <= 2) {
+                days = (long)(v*sign);
+                str = skip_until_space(str);
+                Dprintf("psyco_INTERVAL_cast(): days = %ld, rest = %s\n",
+                        days, str);
+                v = 0.0; sign = 1.0; part = 3;
+            }
+            break;
+
+        case ':':
+            if (part <= 3) {
+                hours = v*sign;
+                Dprintf("psyco_INTERVAL_cast(): hours = %f\n", hours);
+                v = 0.0; part = 4;
+            }
+            else if (part == 4) {
+                minutes = v;
+                Dprintf("psyco_INTERVAL_cast(): minutes = %f\n", minutes);
+                v = 0.0; part = 5;
+            }
+            break;
+
+        default:
+            break;
+        }
+        
+        str++;
+    }
+
+    /* manage last value, be it minutes or seconds */
+    if (part == 4) {
+        minutes = v;
+        Dprintf("psyco_INTERVAL_cast(): minutes = %f\n", minutes);
+    }
+    else if (part == 5) {
+        seconds = v;
+        Dprintf("psyco_INTERVAL_cast(): seconds = %f\n", seconds);
+    }
+
+    /* calculates seconds */
+    if (hours < 0.0) {
+        seconds = - seconds - minutes*60 + hours*3600;
+    }
+    else {
+        seconds += minutes*60 + hours*3600;
+    }
+
+    /* calculates days */
+    days += years*365 + months*30;
+    
+    Dprintf("psyco_INTERVAL_cast(): days = %ld, seconds = %f\n",
+            days, seconds);
+    return mxDateTimeP->DateTimeDelta_FromDaysAndSeconds(days, seconds);
+}
 
 static PyObject *
 psyco_BOOLEAN_cast(PyObject *s)
@@ -120,52 +260,43 @@ psyco_BOOLEAN_cast(PyObject *s)
 static PyObject *
 psyco_BINARY_cast(PyObject *s)
 {
-    char *str;
-    static char numstr[4];
-    char /**tail,*/ *dstptr, *dststr;
-    long len, i;
-    int j;
+    char *str, *dstptr, *dststr;
+    int len, i;
+    PyObject *retstr;
    
     str = PyString_AS_STRING(s);
     len = strlen(str);
-    dststr = (char *)calloc(len, sizeof(char));
+    dststr = (char*)calloc(len, sizeof(char));
     dstptr = dststr;
-    numstr[3] = '\0';
+
+    Py_BEGIN_ALLOW_THREADS;
    
     for (i = 0; i < len; i++) {
-        *dstptr = str[i];
-        if (str[i] != '\\')  dstptr++;
-        else {
-            if ( i++ < len) {
-                *dstptr = 0;
-                for (j = 0; j < 3; j++) {
-                    char c;
-
-                    c = str[i + j];
-                    if ((c & 0370) == '0') {
-                        *dstptr += (c & 7) << (3*(2 - j));
-                    }
-                    else {
-                        *dstptr = str[i];
-                        j = 1;
-                        break;
-                    }
+        if (str[i] == '\\') {
+            if ( ++i < len) {
+                if (str[i] == '\\') {
+                    *dstptr = '\\';
+                }
+                else {
+                    *dstptr = 0;
+                    *dstptr |= (str[i++] & 7) << 6;
+                    *dstptr |= (str[i++] & 7) << 3;
+                    *dstptr |= (str[i] & 7);
                 }
-                dstptr++;
-                i += j - 1;
-                // c = (char)strtol(numstr, &tail, 8);
-                // /* check if xxx was converted */
-                //if (tail - numstr == 3) {
-                //    *dstptr = c;
-                //    i += 2;
-                //}
-                //else i--;
-                //dstptr++;
             }
         }
+        else {
+            *dstptr = str[i];
+        }
+        dstptr++;
     }
+    
+    Py_END_ALLOW_THREADS;
+
     Dprintf("Binary: %s, len: %d\n", dststr, dstptr - dststr);
-    return PyString_FromStringAndSize(dststr, dstptr - dststr);
+    retstr = PyString_FromStringAndSize(dststr, dstptr - dststr);
+    free(dststr);
+    return retstr;
 }
 
 
@@ -185,6 +316,7 @@ psyco_BINARY_cast(PyObject *s)
 
 PyObject *psyco_types;
 PyObject *psyco_default_cast;
+PyObject *psyco_binary_cast;
 
 static int psyco_default_cast_type_DEFAULT[] = {0};
 static psyco_DBAPIInitList psyco_default_cast_type = {
@@ -206,14 +338,22 @@ psyco_init_types(PyObject *md)
      * in the module dictionary */
     for (i = 0; psyco_cast_types[i].name != NULL; i++) {
         psyco_DBAPITypeObject *t;
-        Dprintf("psyco_init_types: initializing %s\n", psyco_cast_types[i].name);
+        Dprintf("psyco_init_types: initializing %s\n",
+                psyco_cast_types[i].name);
         if (!(t = (psyco_DBAPITypeObject *)
               new_psyco_typeobject(&(psyco_cast_types[i])))) return -1;
         if (psyco_add_type((PyObject *)t) != 0) return -1;
-        //Py_INCREF(t->name);
+
         PyDict_SetItem(md, t->name, (PyObject *)t);
+
+        /* export binary object */
+        if (psyco_cast_types[i].values == psyco_cast_types_BINARY) {
+            psyco_binary_cast = (PyObject *)t;
+        }
     }
 
+    assert(psyco_binary_cast != NULL);
+    
     /* create and save a default cast object (but does not register it) */
     psyco_default_cast = new_psyco_typeobject(&psyco_default_cast_type);
     return 0;
@@ -266,9 +406,11 @@ psyco_DBAPITypeObject_coerce(PyObject **pv, PyObject **pw)
     if (PyInt_Check(*pw)) {
         PyObject *coer, *args;
         args = PyTuple_New(1);
+        Py_INCREF(*pw);
         PyTuple_SET_ITEM(args, 0, *pw);
-        coer = psyco_DBAPITypeObject_new(Py_None, args, Py_None);
+        coer = psyco_DBAPITypeObject_new(NULL, args, NULL);
         *pw = coer;
+        Py_DECREF(args);
         Py_INCREF(*pv);
         return 0;
     }
@@ -309,15 +451,18 @@ static PyNumberMethods psyco_DBAPITypeObject_as_number = {
 /* the python object interface for the cursor object */
 
 static int
-psyco_DBAPITypeObject_cmp(psyco_DBAPITypeObject *self, psyco_DBAPITypeObject *v)
+psyco_DBAPITypeObject_cmp(psyco_DBAPITypeObject *self,
+                          psyco_DBAPITypeObject *v)
 {
     int res;
 
-    if (PyObject_Length(v->values) > 1) {
+    if (PyObject_Length(v->values) > 1 && PyObject_Length(self->values) == 1) {
         /* calls itself exchanging the args */
+        Dprintf("psyco_DBAPITypeObject_cmp: swapping args\n");
         return psyco_DBAPITypeObject_cmp(v, self);
     }
-    res = PySequence_In(self->values, PySequence_GetItem(v->values, 0));
+    res = PySequence_Contains(self->values, PyTuple_GET_ITEM(v->values, 0));
+    Dprintf("psyco_DBAPITypeObject_cmp: res = %d\n", res);
     
     if (res < 0) return res;
     else return (res == 1 ? 0 : 1);
@@ -337,14 +482,28 @@ psyco_DBAPITypeObject_getattr(psyco_DBAPITypeObject *self, char *name)
     rv = PyMember_Get((char *)self, psyco_DBAPITypeObject_memberlist, name);
     if (rv) return rv;
     PyErr_Clear();
-    return Py_FindMethod(psyco_DBAPITypeObject_methods, (PyObject *)self, name);
+    return Py_FindMethod(psyco_DBAPITypeObject_methods,
+                         (PyObject *)self, name);
 }
 
 
 static void
 psyco_DBAPITypeObject_destroy(psyco_DBAPITypeObject *self)
 {
-    PyMem_DEL(self);
+    PyObject *name, *cast, *values;
+
+    values = self->values;
+    name = self->name;
+    cast = self->pcast;
+
+    
+    Dprintf("psyco_DBAPITypeObject_destroy: after value refcnt = %d\n",
+            PyTuple_GET_ITEM(self->values, 0)->ob_refcnt);
+    PyObject_Del(self);
+
+    Py_XDECREF(name);
+    Py_XDECREF(values);
+    Py_XDECREF(cast);
     Dprintf("DBAPITypeObject at %p destroyed\n", self);
 }
 
@@ -357,15 +516,12 @@ psyco_DBAPITypeObject_call(PyObject *self, PyObject *args, PyObject *keywds)
     
     PyObject *string, *res;
     psyco_DBAPITypeObject *me = (psyco_DBAPITypeObject *)self;
-    static PyObject *arg = NULL;
     
     Dprintf("psyco_DBAPITypeObject_call: self = %p, args = %p\n", self, args);
     
     if (!PyArg_ParseTuple(args, "O!", &PyString_Type, &string)) {
         return NULL;
     }
-
-    if (arg == NULL) arg = PyTuple_New(1);
     
     Dprintf("psyco_DBAPITypeObject_call: string argument at %p, refcnt = %d\n",
             string, string->ob_refcnt);
@@ -375,10 +531,14 @@ psyco_DBAPITypeObject_call(PyObject *self, PyObject *args, PyObject *keywds)
         res = me->ccast(string);
     }
     else if (me->pcast) {
+        PyObject *arg = PyTuple_New(1);
+        
         Dprintf("psyco_DBAPITypeObject_call: calling python callable\n");
         PyTuple_SET_ITEM(arg, 0, string);
+        Py_INCREF(string);
         res = PyObject_CallObject(me->pcast, arg);
         Dprintf("psyco_DBAPITypeObject_call: res = %p\n", res);
+        Py_DECREF(arg);
     }
     else {
         Py_INCREF(Py_None);
@@ -448,7 +608,12 @@ psyco_DBAPITypeObject_new(PyObject *name, PyObject *values, PyObject *cast)
         obj->pcast = cast;
     }
     obj->ccast = NULL;
-
+    
+    Dprintf("psyco_DBAPITypeObject_new: name = %p, values = %p, "
+            "pcast = %p, ccast = %p\n",
+            obj->name, obj->values, obj->pcast, obj->ccast);
+    Dprintf("psyco_DBAPITypeObject_new: values refcnt = %d\n",
+            values->ob_refcnt);
     Dprintf("psyco_DBAPITypeObject_new: object created at %p\n", obj);
     return (PyObject *)obj;
 }
@@ -495,7 +660,8 @@ new_psyco_typeobject(psyco_DBAPIInitList *type)
     }
     
     obj = (psyco_DBAPITypeObject *)
-        psyco_DBAPITypeObject_new(PyString_FromString(type->name), tuple, NULL);
+        psyco_DBAPITypeObject_new(PyString_FromString(type->name),
+                                  tuple, NULL);
     
     if (obj) {
         obj->ccast = type->cast;
